@@ -332,9 +332,38 @@ test_that("NAs are handled correctly in point_interval", {
     mean_hdci(c(0:6, 1:5, 2:4, 2, NA), .width = .6, na.rm = FALSE),
     data.frame(y = NA_real_, ymin = NA_real_, ymax = NA_real_, .width = 0.6, .point = "mean", .interval = "hdci", stringsAsFactors = FALSE)
   )
+
+  expect_equal(
+    mean_ll(c(0:6, 1:5, 2:4, 2, NA), .width = .6, na.rm = FALSE),
+    data.frame(y = NA_real_, ymin = NA_real_, ymax = NA_real_, .width = 0.6, .point = "mean", .interval = "ll", stringsAsFactors = FALSE)
+  )
+
+  expect_equal(
+    mean_ul(c(0:6, 1:5, 2:4, 2, NA), .width = .6, na.rm = FALSE),
+    data.frame(y = NA_real_, ymin = NA_real_, ymax = NA_real_, .width = 0.6, .point = "mean", .interval = "ul", stringsAsFactors = FALSE)
+  )
+
 })
 
 
+# upper/lower limits (ul/ll) ----------------------------------------------
+
+test_that("ll and ul work", {
+  df = data.frame(x = ppoints(100, a = 1))
+
+  ref = tibble(x = 0.5, .lower = c(0.25, 0), .upper = 1, .width = c(0.75, 1), .point = "mean", .interval = "ll")
+
+  expect_equal(mean_ll(df, x, .width = c(.75, 1)), ref)
+  expect_equal(median_ll(df, x, .width = c(.75, 1)), mutate(ref, .point = "median"))
+  expect_equal(mode_ll(df, x, .width = c(.75, 1)) %>% mutate(x = round(x, 2)), mutate(ref, .point = "mode"))
+
+  ref = tibble(x = 0.5, .lower = 0, .upper = c(0.75, 1), .width = c(0.75, 1), .point = "mean", .interval = "ul")
+
+  expect_equal(mean_ul(df, x, .width = c(.75, 1)), ref)
+  expect_equal(median_ul(df, x, .width = c(.75, 1)), mutate(ref, .point = "median"))
+  expect_equal(mode_ul(df, x, .width = c(.75, 1)) %>% mutate(x = round(x, 2)), mutate(ref, .point = "mode"))
+})
+
 # rvars -------------------------------------------------
 
 test_that("pointintervals work on rvars", {
@@ -356,12 +385,133 @@ test_that("pointintervals work on rvars", {
   )
 })
 
+test_that("non-scalar rvars throw appropriate warnings", {
+  skip_if_not_installed("posterior")
+
+  x = posterior::rvar(matrix(1:6, nrow = 2))
+  expect_error(hdi(x), "HDI for non-scalar rvars is not implemented")
+  expect_error(hdci(x), "HDCI for non-scalar rvars is not implemented")
+})
+
+test_that("point_interval works on NA rvars", {
+  skip_if_not_installed("posterior")
+
+  ref = tibble(
+    .value = NA_real_,
+    .lower = NA_real_,
+    .upper = NA_real_,
+    .width = 0.95
+  )
+
+  x = posterior::rvar(NA_real_)
+  expect_equal(median_qi(x), mutate(ref, .point = "median", .interval = "qi"))
+  expect_equal(mean_hdi(x), mutate(ref, .point = "mean", .interval = "hdi"))
+  expect_equal(mode_hdci(x), mutate(ref, .point = "mode", .interval = "hdci"))
+})
+
+test_that("multivariate rvars work", {
+  skip_if_not_installed("posterior")
+
+  x = c(
+    posterior::rvar(c(qnorm(ppoints(50)), qnorm(ppoints(50), 5))),
+    posterior::rvar(c(qnorm(ppoints(100), 3))),
+    posterior::rvar(c(qnorm(ppoints(100), 2))),
+    posterior::rvar(c(qnorm(ppoints(100), 4)))
+  )
+  dim(x) = c(2,2)
+
+  df = tibble(g = c("a", "b"), x = x)
+
+
+  # build qi ref
+  qis_50 = rbind(
+    qi(posterior::draws_of(x[1,1]), .width = .5),
+    qi(posterior::draws_of(x[1,2]), .width = .5),
+    qi(posterior::draws_of(x[2,1]), .width = .5),
+    qi(posterior::draws_of(x[2,2]), .width = .5)
+  )
+  qis_90 = rbind(
+    qi(posterior::draws_of(x[1,1]), .width = .9),
+    qi(posterior::draws_of(x[1,2]), .width = .9),
+    qi(posterior::draws_of(x[2,1]), .width = .9),
+    qi(posterior::draws_of(x[2,2]), .width = .9)
+  )
+
+  expect_equal(dim(qis_50), c(4,2))
+  expect_equal(dim(qis_90), c(4,2))
+
+  ref = tibble(
+    g = rep(c("a", "a", "b", "b"), 2),
+    x = rep(c(2.5, 2, 3, 4), 2),
+    .index = rep(1:2, 4),
+    .lower = c(qis_50[,1], qis_90[,1]),
+    .upper = c(qis_50[,2], qis_90[,2]),
+    .width = rep(c(0.5, 0.9), each = 4),
+    .point = "median",
+    .interval = "qi"
+  )
+
+  expect_equal(median_qi(df, x, .width = c(.5, .9)), ref)
+
+
+  # build hdi ref
+  hdis_50 = rbind(
+    hdi(posterior::draws_of(x[1,1]), .width = .5),
+    hdi(posterior::draws_of(x[1,2]), .width = .5),
+    hdi(posterior::draws_of(x[2,1]), .width = .5),
+    hdi(posterior::draws_of(x[2,2]), .width = .5)
+  )
+  hdis_90 = rbind(
+    hdi(posterior::draws_of(x[1,1]), .width = .9),
+    hdi(posterior::draws_of(x[1,2]), .width = .9),
+    hdi(posterior::draws_of(x[2,1]), .width = .9),
+    hdi(posterior::draws_of(x[2,2]), .width = .9)
+  )
+
+  expect_equal(dim(hdis_50), c(5,2))
+  expect_equal(dim(hdis_90), c(5,2))
+
+  ref = tibble(
+    g = rep(c("a", "a", "a", "b", "b"), 2),
+    x = rep(c(2.5, 2.5, 2, 3, 4), 2),
+    .index = rep(c(1,1,2,1,2), 2),
+    .lower = c(hdis_50[,1], hdis_90[,1]),
+    .upper = c(hdis_50[,2], hdis_90[,2]),
+    .width = rep(c(0.5, 0.9), each = 5),
+    .point = "median",
+    .interval = "hdi"
+  )
+
+  expect_equal(median_hdi(df, x, .width = c(.5, .9)), ref)
+
+
+  # > 2 dims
+  x_draws = array(c(
+    rep(qi(ppoints(100, a = 1)), 12) + rep(1:12, each = 100)
+  ), dim = c(100, 2, 3, 2))
+  df = tibble(i = 1:2, x = posterior::rvar(x_draws))
+
+  .index = paste0(rep(1:3, 4), ",", rep(1:2, each = 3, times = 2))
+  .index = ordered(.index, levels = paste0(rep(1:3, 2), ",", rep(1:2, each = 3)))
+  .x = c(seq(1, 11, by = 2), seq(2, 12, by = 2))
+  ref = tibble(
+    i = rep(1:2, each = 6),
+    x = .x + 0.5,
+    .index = .index,
+    .lower = .x + .025,
+    .upper = .x + .975,
+    .width = .95,
+    .point = "median",
+    .interval = "qi"
+  )
+
+  expect_equal(median_qi(df, x), ref)
+})
+
 
 # distributional objects --------------------------------------------------
 
 test_that("pointintervals work on distributional objects", {
-  skip_if_not_installed("distributional", "0.2.2.9000")
-
   x = dist_gamma(1:2,2:3)
 
   expect_equal(
@@ -378,3 +528,88 @@ test_that("pointintervals work on distributional objects", {
     tolerance = 1e-05
   )
 })
+
+test_that("Mode on dist_sample uses the numeric method", {
+  x_values = dgamma(ppoints(100), 2, 2)
+  x = dist_sample(list(x_values))
+
+  expect_equal(Mode(x), Mode(x_values))
+})
+
+test_that("non-scalar distributions throw appropriate warnings", {
+  x = dist_normal(0:1)
+  expect_error(hdi(x), "HDI for non-scalar distribution objects is not implemented")
+  expect_error(hdci(x), "HDCI for non-scalar distribution objects is not implemented")
+})
+
+test_that("multivariate distributions throw appropriate warnings", {
+  skip_if_not_installed("mvtnorm") # needed by distributional::support for dist_mvnorm
+
+  x = dist_multivariate_normal(list(0:1), list(diag(2)))
+  expect_error(hdi(x), "HDI for multivariate distribution objects is not implemented")
+  expect_error(hdci(x), "HDCI for multivariate distribution objects is not implemented")
+})
+
+test_that("point_interval works on NA dists", {
+  ref = tibble(
+    .value = NA_real_,
+    .lower = NA_real_,
+    .upper = NA_real_,
+    .width = 0.95
+  )
+
+  x = dist_missing()
+  expect_equal(median_qi(x), mutate(ref, .point = "median", .interval = "qi"))
+  expect_equal(mean_hdi(x), mutate(ref, .point = "mean", .interval = "hdi"))
+  expect_equal(mode_hdci(x), mutate(ref, .point = "mode", .interval = "hdci"))
+})
+
+test_that("multivariate distributions work", {
+
+  x = c(dist_multivariate_normal(list(1:3), list(diag(3))), dist_normal(0,0.5))
+
+  df = tibble(g = c("a", "b"), x = x)
+
+  ref = tibble(
+    g = rep(c("a", "a", "a", "b"), 2),
+    x = rep(c(1, 2, 3, 0), 2),
+    .index = rep(c(1, 2, 3, NA), 2),
+    .lower = c(
+      qnorm(0.25, c(1, 2, 3, 0), c(1, 1, 1, 0.5)),
+      qnorm(0.05, c(1, 2, 3, 0), c(1, 1, 1, 0.5))
+    ),
+    .upper = c(
+      qnorm(0.75, c(1, 2, 3, 0), c(1, 1, 1, 0.5)),
+      qnorm(0.95, c(1, 2, 3, 0), c(1, 1, 1, 0.5))
+    ),
+    .width = rep(c(0.5, 0.9), each = 4),
+    .point = "median",
+    .interval = "qi"
+  )
+
+  expect_equal(median_qi(df, x, .width = c(.5, .9)), ref)
+})
+
+test_that("flattened indices retain index order", {
+  skip_if_no_vdiffr()
+
+  vdiffr::expect_doppelganger("flattened indices with geom_pointinterval",
+    tibble(x = dist_multivariate_normal(list(c(1:10)), list(diag(10)))) %>%
+      median_qi(x, .width = c(.66, .95)) %>%
+      ggplot(aes(x, xmin = .lower, xmax = .upper, y = .index)) +
+      geom_pointinterval()
+  )
+
+  vdiffr::expect_doppelganger("flattened indices with stat_pointinterval",
+    tibble(
+      x = c(
+        dist_multivariate_normal(list(c(1:10)), list(diag(10))),
+        dist_normal()
+      ),
+      y = c("a","b")
+    )  %>%
+    ggplot(aes(xdist = x, y = y, group = stat(.index))) +
+    stat_pointinterval(position = "dodge")
+  )
+
+})
